@@ -24,10 +24,14 @@
  * 	http://en.wikipedia.org/wiki/Sample_Font_Displays_In_Other_Languages
  * 	Fetched on 2008-08-19
  *
+ * WP
+ *      Wikipedia, Article about the language
+ *      Fetched on 2020-09-08
+ *
  * GLASS
  * 	Kermit project's "I Can Eat Glass" list, also available in pango-view/
  * 	http://www.columbia.edu/kermit/utf8.html#glass
- * 	Fetched on 2008-08-19
+ * 	Fetched on 2008-08-19, updates on 2020-09-08
  *
  * KERMIT
  * 	Kermit project's Quick-Brown-Fox equivalents for other languages
@@ -71,6 +75,12 @@ LANGUAGE(
 	 "Gvxam mincetu apocikvyeh: ñizol ce mamvj ka raq kuse bafkeh mew."
 	 /* Tale under the full moon: the chief chemamull and the clay old woman at the lake/sea. */
 	)
+LANGUAGE(
+	 as	/* Assamese */,
+	 WP,
+	 "সুপ্ৰভাত আপোনাক লগ পাই ভাল লাগিল।"
+	 /* Good morning, nice to meet you. */
+	)
 LANGUAGE(
 	 bar	/* Bavarian */,
 	 GLASS,
@@ -143,6 +153,18 @@ LANGUAGE(
 	 "Zwölf Boxkämpfer jagen Viktor quer über den großen Sylter Deich."
 	 /* Twelve boxing fighters drive Viktor over the great. */
 	)
+LANGUAGE(
+	 doi	/* Dogri */,
+	 WP,
+	 "अ'ऊं बमार आँ, मिगी डॉक्टर दी लोड़ ऐ।"
+	 /* I'm sick, I need a doctor. */
+	)
+LANGUAGE(
+         dv     /* Maldivian */,
+         WP,
+         "މާއްދާ 1 – ހުރިހާ އިންސާނުން ވެސް އުފަންވަނީ، ދަރަޖަ އާއި ޙައްޤު ތަކުގައި މިނިވަންކަމާއި ހަމަހަމަކަން ލިބިގެންވާ ބައެއްގެ ގޮތުގައެވެ."
+         /* Beginning of UDHR */
+        )
 LANGUAGE(
 	 el	/* Greek */,
 	 WP-SFD,
@@ -263,9 +285,9 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 hr	/* Croatian */,
-	 GLASS,
-	 "Ja mogu jesti staklo i ne boli me."
-	 /* I can eat glass and it doesn't hurt me. */
+	 MISC,
+	 "Deblji krojač: zgužvah smeđ filc u tanjušni džepić."
+	 /* A fatter taylor: I’ve crumpled a brown felt in a slim pocket. */
 	)
 LANGUAGE(
 	 hu	/* Hungarian */,
@@ -319,6 +341,12 @@ LANGUAGE(
 	 "მინას ვჭამ და არა მტკივა."
 	 /* I can eat glass and it doesn't hurt me. */
 	)
+LANGUAGE(
+         km     /* Khmer */,
+         GLASS,
+         "ខ្ញុំអាចញុំកញ្ចក់បាន ដោយគ្មានបញ្ហារ"
+	 /* I can eat glass and it doesn't hurt me. */
+        )
 LANGUAGE(
 	 kn	/* Kannada */,
 	 GLASS,
@@ -331,6 +359,12 @@ LANGUAGE(
 	 "다람쥐 헌 쳇바퀴에 타고파"
 	 /* I Wanna ride on the chipmunk's old hamster wheel. */
 	)
+LANGUAGE(
+	 ks	/* Kashmiri */,
+	 WP,
+	 "ब’ छस न’ ज़ानान, तोहय हेकिवा दुबार’ वॅनिथ ।"
+	 /* I don't understand, Please say that again. */
+	)
 LANGUAGE(
 	 kw	/* Cornish */,
 	 GLASS,
@@ -342,6 +376,12 @@ LANGUAGE(
 	 WP-PANG,
 	 "Sic surgens, dux, zelotypos quam karus haberis"
 	)
+LANGUAGE(
+         lo     /* Lao */,
+         GLASS,
+         "ຂອ້ຍກິນແກ້ວໄດ້ໂດຍທີ່ມັນບໍ່ໄດ້ເຮັດໃຫ້ຂອ້ຍເຈັບ"
+	 /* I can eat glass and it doesn't hurt me. */
+        )
 LANGUAGE(
 	 lt	/* Lithuanian */,
 	 WP-PANG,
@@ -354,6 +394,12 @@ LANGUAGE(
 	 "Sarkanās jūrascūciņas peld pa jūru."
 	 /* Red seapigs swim in the sea. */
 	)
+LANGUAGE(
+	 mai	/* Maithili */,
+	 WP,
+	 "हम अहाँ के बिना कुछ नै छिय"
+	 /* I am nothing without you */
+	)
 LANGUAGE(
 	 map	/* Marquesan */,
 	 GLASS,
@@ -390,6 +436,12 @@ LANGUAGE(
 	 "Saya boleh makan kaca dan ia tidak mencederakan saya."
 	 /* I can eat glass and it doesn't hurt me. */
 	)
+LANGUAGE(
+         my     /* Burmese */,
+         WP,
+         "ဘာသာပြန်နှင့် စာပေပြုစုရေး ကော်မရှင်"
+         /* Literary and Translation Commission */
+        )
 LANGUAGE(
 	 nap	/* Neapolitan */,
 	 GLASS,
@@ -401,6 +453,12 @@ LANGUAGE(
 	 GSPECI,
 	 "Vår sære Zulu fra badeøya spilte jo whist og quickstep i min taxi."
 	)
+LANGUAGE(
+	 ne	/* Nepali */,
+	 WP,
+	 "के तपाईँ मसँग नाच्न चाहनुहुन्छ ?"
+	 /* Would you like to dance with me? */
+	)
 LANGUAGE(
 	 nl	/* Dutch */,
 	 WP-SFD,
@@ -462,9 +520,9 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 pt-br	/* Brazilian Portuguese */,
-	 WP-PANG,
-	 "À noite, vovô Kowalsky vê o ímã cair no pé do pingüim queixoso e vovó põe açúcar no chá de tâmaras do jabuti feliz."
-	 /* At night, grandpa Kowalsky sees the magnet falling in the complaining penguin's foot and grandma puts sugar in the happy tortoise's date tea.*/
+	 MISC, /* https://gitlab.gnome.org/GNOME/pango/-/issues/547 */
+         "Vejo galã sexy pôr quinze kiwis à força em baú achatado."
+         /* I see a sexy hunk forcefully stuff fifteen kiwis in a shallow trunk */
 	)
 LANGUAGE(
 	 ro	/* Romanian */,
@@ -490,6 +548,17 @@ LANGUAGE(
 	 "Puotsu mangiari u vitru, nun mi fa mali."
 	 /* I can eat glass and it doesn't hurt me. */
 	)
+LANGUAGE(
+	 sd	/* Sindhi */,
+	 WP,
+	 "توهان ملي ڏاڍو سٺو لڳو"
+	 /* Pleased to meet you */
+	)
+LANGUAGE(
+         si     /* Sinhalese */,
+         WP,
+         "මනොපුබ්‌බඞ්‌ගමා ධම්‌මා, මනොසෙට්‌ඨා මනොමයා; මනසා චෙ පදුට්‌ඨෙන, භාසති වා කරොති වා; තතො නං දුක්‌ඛමන්‌වෙති, චක්‌කංව වහතො පදං."
+        )
 LANGUAGE(
 	 sk	/* Slovak */,
 	 KERMIT,
