@@ -22,14 +22,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:win32-fonts
- * @short_description:Font handling and rendering on Windows
- * @title:Win32 Fonts and Rendering
- *
- * The macros and functions in this section are used to access fonts natively on
- * Win32 systems and to render text in conjunction with Win32 APIs.
- */
 #include "config.h"
 
 #include <string.h>
@@ -37,9 +29,16 @@
 #include <glib.h>
 #include <hb.h>
 
+#ifdef USE_HB_GDI
+#include <hb-gdi.h>
+#endif
+
+#include <hb-ot.h>
+
 #include "pango-impl-utils.h"
 #include "pangowin32.h"
 #include "pangowin32-private.h"
+#include "pango-coverage-private.h"
 
 #define MAX_FREED_FONTS 256
 
@@ -64,6 +63,7 @@ static gboolean pango_win32_font_real_select_font      (PangoFont *font,
 							HDC        hdc);
 static void     pango_win32_font_real_done_font        (PangoFont *font);
 static double   pango_win32_font_real_get_metrics_factor (PangoFont *font);
+static gboolean pango_win32_font_is_hinted             (PangoFont *font);
 
 static void                  pango_win32_get_item_properties    (PangoItem        *item,
 								 PangoUnderline   *uline,
@@ -108,13 +108,13 @@ _pango_win32_font_get_hfont (PangoFont *font)
 /**
  * pango_win32_get_context:
  *
- * Retrieves a #PangoContext appropriate for rendering with Windows fonts.
+ * Retrieves a `PangoContext` appropriate for rendering with Windows fonts.
  *
- * Return value: the new #PangoContext
+ * Return value: the new `PangoContext`
  *
- * Deprecated: 1.22: Use pango_win32_font_map_for_display() followed by
- * pango_font_map_create_context() instead.
- **/
+ * Deprecated: 1.22: Use [func@Pango.Win32FontMap.for_display] followed by
+ * [method@Pango.FontMap.create_context] instead.
+ */
 PangoContext *
 pango_win32_get_context (void)
 {
@@ -133,7 +133,16 @@ _pango_win32_font_init (PangoWin32Font *win32font)
   win32font->glyph_info = g_hash_table_new_full (NULL, NULL, NULL, g_free);
 }
 
-static GPrivate display_dc_key = G_PRIVATE_INIT ((GDestroyNotify) DeleteDC);
+static void
+_delete_dc (HDC dc)
+{
+  /* Don't pass DeleteDC func pointer to the GDestroyNotify.
+   * 32bit build requires matching calling convention (__cdecl vs __stdcall) */
+  DeleteDC (dc);
+}
+
+static GPrivate display_dc_key = G_PRIVATE_INIT ((GDestroyNotify) _delete_dc);
+static GPrivate dwrite_items = G_PRIVATE_INIT ((GDestroyNotify) pango_win32_dwrite_items_destroy);
 
 HDC
 _pango_win32_get_display_dc (void)
@@ -159,9 +168,26 @@ _pango_win32_get_display_dc (void)
 #endif
     }
 
+  /* ensure DirectWrite is initialized */
+  pango_win32_get_direct_write_items ();
+
   return hdc;
 }
 
+PangoWin32DWriteItems *
+pango_win32_get_direct_write_items (void)
+{
+  PangoWin32DWriteItems *items = g_private_get (&dwrite_items);
+
+  if (items == NULL)
+    {
+      items = pango_win32_init_direct_write ();
+      g_private_set (&dwrite_items, items);
+    }
+
+  return items;
+}
+
 /**
  * pango_win32_get_dc:
  *
@@ -195,6 +221,7 @@ _pango_win32_font_class_init (PangoWin32FontClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   PangoFontClass *font_class = PANGO_FONT_CLASS (class);
+  PangoFontClassPrivate *pclass;
 
   object_class->finalize = pango_win32_font_finalize;
   object_class->dispose = pango_win32_font_dispose;
@@ -211,6 +238,9 @@ _pango_win32_font_class_init (PangoWin32FontClass *class)
   class->done_font = pango_win32_font_real_done_font;
   class->get_metrics_factor = pango_win32_font_real_get_metrics_factor;
 
+  pclass = g_type_class_get_private ((GTypeClass *) class, PANGO_TYPE_FONT);
+  pclass->is_hinted = pango_win32_font_is_hinted;
+
   _pango_win32_get_display_dc ();
 }
 
@@ -222,7 +252,7 @@ _pango_win32_font_class_init (PangoWin32FontClass *class)
  * @x:       the x position of start of string (in pixels)
  * @y:       the y position of baseline (in pixels)
  *
- * Render a #PangoGlyphString onto a Windows DC
+ * Render a `PangoGlyphString` onto a Windows DC
  */
 void
 pango_win32_render (HDC               hdc,
@@ -378,23 +408,24 @@ pango_win32_render (HDC               hdc,
 
 /**
  * pango_win32_render_transformed:
- * @hdc:     a windows device context
- * @matrix:  (nullable): a #PangoMatrix, or %NULL to use an identity
- *           transformation
- * @font:    the font in which to draw the string
- * @glyphs:  the glyph string to draw
- * @x:       the x position of the start of the string (in Pango
- *           units in user space coordinates)
- * @y:       the y position of the baseline (in Pango units
- *           in user space coordinates)
+ * @hdc: a windows device context
+ * @matrix: (nullable): a `PangoMatrix`
+ * @font: the font in which to draw the string
+ * @glyphs: the glyph string to draw
+ * @x: the x position of the start of the string (in Pango
+ *   units in user space coordinates)
+ * @y: the y position of the baseline (in Pango units
+ *   in user space coordinates)
  *
- * Renders a #PangoGlyphString onto a windows DC, possibly
+ * Renders a `PangoGlyphString` onto a windows DC, possibly
  * transforming the layed-out coordinates through a transformation
- * matrix. Note that the transformation matrix for @font is not
+ * matrix.
+ *
+ * Note that the transformation matrix for @font is not
  * changed, so to produce correct rendering results, the @font
- * must have been loaded using a #PangoContext with an identical
+ * must have been loaded using a `PangoContext` with an identical
  * transformation matrix to that passed in to this function.
- **/
+ */
 void
 pango_win32_render_transformed (HDC                hdc,
 				const PangoMatrix *matrix,
@@ -603,7 +634,7 @@ pango_win32_font_get_metrics (PangoFont     *font,
 	  else
 	    metrics->approximate_digit_width = metrics->approximate_char_width;
 
-	  pango_coverage_unref (coverage);
+	  g_object_unref (coverage);
 
 	  /* FIXME: Should get the real values from the TrueType font file */
 	  metrics->underline_position = -2 * PANGO_SCALE;
@@ -654,9 +685,17 @@ pango_win32_font_real_get_metrics_factor (PangoFont *font)
   return PANGO_SCALE;
 }
 
+static gboolean
+pango_win32_font_is_hinted (PangoFont *font)
+{
+  g_return_val_if_fail (PANGO_WIN32_IS_FONT (font), FALSE);
+
+  return PANGO_WIN32_FONT (font)->is_hinted;
+}
+
 /**
  * pango_win32_font_logfont:
- * @font: a #PangoFont which must be from the Win32 backend
+ * @font: a `PangoFont` which must be from the Win32 backend
  *
  * Determine the LOGFONTA struct for the specified font. Note that
  * Pango internally uses LOGFONTW structs, so if converting the UTF-16
@@ -667,8 +706,8 @@ pango_win32_font_real_get_metrics_factor (PangoFont *font)
  * to come across fonts with odd names.
  *
  * Return value: A newly allocated LOGFONTA struct. It must be
- * freed with g_free().
- **/
+ *   freed with g_free().
+ */
 LOGFONTA *
 pango_win32_font_logfont (PangoFont *font)
 {
@@ -692,12 +731,12 @@ pango_win32_font_logfont (PangoFont *font)
 
 /**
  * pango_win32_font_logfontw:
- * @font: a #PangoFont which must be from the Win32 backend
- * 
+ * @font: a `PangoFont` which must be from the Win32 backend
+ *
  * Determine the LOGFONTW struct for the specified font.
- * 
+ *
  * Return value: A newly allocated LOGFONTW struct. It must be
- * freed with g_free().
+ *   freed with g_free().
  *
  * Since: 1.16
  **/
@@ -718,21 +757,22 @@ pango_win32_font_logfontw (PangoFont *font)
 
 /**
  * pango_win32_font_select_font:
- * @font: a #PangoFont from the Win32 backend
+ * @font: a `PangoFont` from the Win32 backend
  * @hdc: a windows device context
  *
  * Selects the font into the specified DC and changes the mapping mode
  * and world transformation of the DC appropriately for the font.
+ *
  * You may want to surround the use of this function with calls
- * to SaveDC() and RestoreDC(). Call pango_win32_font_done_font() when
+ * to SaveDC() and RestoreDC(). Call [method@Pango.Win32Font.done_font[ when
  * you are done using the DC to release allocated resources.
  *
- * See pango_win32_font_get_metrics_factor() for information about
+ * See [method@Pango.Win32Font.get_metrics_factor] for information about
  * converting from the coordinate space used by this function
  * into Pango units.
  *
  * Return value: %TRUE if the operation succeeded.
- **/
+ */
 gboolean
 pango_win32_font_select_font (PangoFont *font,
 			      HDC        hdc)
@@ -744,10 +784,10 @@ pango_win32_font_select_font (PangoFont *font,
 
 /**
  * pango_win32_font_done_font:
- * @font: a #PangoFont from the win32 backend
+ * @font: a `PangoFont` from the win32 backend
  *
- * Releases any resources allocated by pango_win32_font_done_font()
- **/
+ * Releases any resources allocated by [method@Pango.Win32Font.select_font].
+ */
 void
 pango_win32_font_done_font (PangoFont *font)
 {
@@ -758,15 +798,15 @@ pango_win32_font_done_font (PangoFont *font)
 
 /**
  * pango_win32_font_get_metrics_factor:
- * @font: a #PangoFont from the win32 backend
+ * @font: a `PangoFont` from the win32 backend
  *
  * Returns the scale factor from logical units in the coordinate
- * space used by pango_win32_font_select_font() to Pango units
- * in user space.
+ * space used by [method@Pango.Win32Font.select_font] to Pango
+ * units in user space.
  *
  * Return value: factor to multiply logical units by to get Pango
- *               units.
- **/
+ *   units.
+ */
 double
 pango_win32_font_get_metrics_factor (PangoFont *font)
 {
@@ -826,12 +866,8 @@ pango_win32_font_finalize (GObject *object)
 {
   PangoWin32Font *win32font = (PangoWin32Font *)object;
   PangoWin32FontCache *cache = pango_win32_font_map_get_font_cache (win32font->fontmap);
-  PangoWin32Font *fontmap;
 
-  if (G_UNLIKELY (!cache))
-    return;
-
-  if (win32font->hfont != NULL)
+  if (cache != NULL && win32font->hfont != NULL)
     pango_win32_font_cache_unload (cache, win32font->hfont);
 
   g_slist_foreach (win32font->metrics_by_lang, (GFunc)free_metrics_info, NULL);
@@ -842,12 +878,10 @@ pango_win32_font_finalize (GObject *object)
 
   g_hash_table_destroy (win32font->glyph_info);
 
-  fontmap = g_weak_ref_get ((GWeakRef *) &win32font->fontmap);
-  if (fontmap)
-  {
-    g_object_remove_weak_pointer (G_OBJECT (win32font->fontmap), (gpointer *) (gpointer) &win32font->fontmap);
-    g_object_unref (fontmap);
-  }
+  if (win32font->fontmap)
+    g_object_remove_weak_pointer (G_OBJECT (win32font->fontmap), (gpointer *) &win32font->fontmap);
+
+  g_free (win32font->variations);
 
   G_OBJECT_CLASS (_pango_win32_font_parent_class)->finalize (object);
 }
@@ -857,9 +891,14 @@ pango_win32_font_describe (PangoFont *font)
 {
   PangoFontDescription *desc;
   PangoWin32Font *win32font = PANGO_WIN32_FONT (font);
+  int size;
 
   desc = pango_font_description_copy (win32font->win32face->description);
-  pango_font_description_set_size (desc, win32font->size / (PANGO_SCALE / PANGO_WIN32_FONT_MAP (win32font->fontmap)->resolution));
+  size = (int) (0.5 + win32font->size * PANGO_WIN32_FONT_MAP (win32font->fontmap)->resolution / PANGO_SCALE);
+  pango_font_description_set_size (desc, size);
+
+  if (win32font->variations)
+    pango_font_description_set_variations (desc, win32font->variations);
 
   return desc;
 }
@@ -873,6 +912,9 @@ pango_win32_font_describe_absolute (PangoFont *font)
   desc = pango_font_description_copy (win32font->win32face->description);
   pango_font_description_set_absolute_size (desc, win32font->size);
 
+  if (win32font->variations)
+    pango_font_description_set_variations (desc, win32font->variations);
+
   return desc;
 }
 
@@ -891,21 +933,20 @@ pango_win32_font_get_coverage (PangoFont     *font,
       hb_codepoint_t ch = HB_SET_VALUE_INVALID;
 
       hb_face_collect_unicodes (hb_face, chars);
-      while (hb_set_next(chars, &ch))
-        pango_coverage_set (coverage, ch, PANGO_COVERAGE_EXACT);
 
-      win32face->coverage = pango_coverage_ref (coverage);
+      coverage->chars = chars;
+      win32face->coverage = coverage;
     }
 
-  return pango_coverage_ref (win32face->coverage);
+  return g_object_ref (win32face->coverage);
 }
 
 /* Utility functions */
 
 /**
  * pango_win32_get_unknown_glyph:
- * @font: a #PangoFont
- * @wc: the Unicode character for which a glyph is needed.
+ * @font: a `PangoFont`
+ * @wc: the Unicode character for which a glyph is needed
  *
  * Returns the index of a glyph suitable for drawing @wc as an
  * unknown character.
@@ -913,7 +954,7 @@ pango_win32_font_get_coverage (PangoFont     *font,
  * Use PANGO_GET_UNKNOWN_GLYPH() instead.
  *
  * Return value: a glyph index into @font
- **/
+ */
 PangoGlyph
 pango_win32_get_unknown_glyph (PangoFont *font,
 			       gunichar   wc)
@@ -923,14 +964,15 @@ pango_win32_get_unknown_glyph (PangoFont *font,
 
 /**
  * pango_win32_render_layout_line:
- * @hdc:       DC to use for drawing
- * @line:      a #PangoLayoutLine
- * @x:         the x position of start of string (in pixels)
- * @y:         the y position of baseline (in pixels)
+ * @hdc: DC to use for drawing
+ * @line: a `PangoLayoutLine`
+ * @x: the x position of start of string (in pixels)
+ * @y: the y position of baseline (in pixels)
  *
- * Render a #PangoLayoutLine onto a device context. For underlining to
- * work property the text alignment of the DC should have TA_BASELINE
- * and TA_LEFT.
+ * Render a `PangoLayoutLine` onto a device context.
+ *
+ * For underlining to work property the text alignment
+ * of the DC should have TA_BASELINE and TA_LEFT.
  */
 void
 pango_win32_render_layout_line (HDC              hdc,
@@ -1063,6 +1105,8 @@ pango_win32_render_layout_line (HDC              hdc,
 	case PANGO_UNDERLINE_ERROR_LINE:
           g_warning ("Underline value %d not implemented", uline);
           break;
+        default:
+          g_assert_not_reached ();
 	}
 
       if (uline != PANGO_UNDERLINE_NONE)
@@ -1079,12 +1123,12 @@ pango_win32_render_layout_line (HDC              hdc,
 
 /**
  * pango_win32_render_layout:
- * @hdc:       HDC to use for drawing
- * @layout:    a #PangoLayout
- * @x:         the X position of the left of the layout (in pixels)
- * @y:         the Y position of the top of the layout (in pixels)
+ * @hdc: HDC to use for drawing
+ * @layout: a `PangoLayout`
+ * @x: the X position of the left of the layout (in pixels)
+ * @y: the Y position of the top of the layout (in pixels)
  *
- * Render a #PangoLayoutLine onto an X drawable
+ * Render a `PangoLayoutLine` onto an HDC.
  */
 void
 pango_win32_render_layout (HDC          hdc,
@@ -1145,7 +1189,7 @@ pango_win32_get_item_properties (PangoItem      *item,
     {
       PangoAttribute *attr = tmp_list->data;
 
-      switch (attr->klass->type)
+      switch ((int) attr->klass->type)
 	{
 	case PANGO_ATTR_UNDERLINE:
 	  if (uline)
@@ -1185,14 +1229,14 @@ pango_win32_get_item_properties (PangoItem      *item,
 
 /**
  * pango_win32_font_get_glyph_index:
- * @font: a #PangoFont.
- * @wc: a Unicode character.
+ * @font: a `PangoFont`
+ * @wc: a Unicode character
  *
  * Obtains the index of the glyph for @wc in @font, or 0, if not
  * covered.
  *
  * Return value: the glyph index for @wc.
- **/
+ */
 gint
 pango_win32_font_get_glyph_index (PangoFont *font,
 				  gunichar   wc)
@@ -1219,7 +1263,12 @@ static inline guint32 hb_gdi_uint32_swap (const guint32 v)
 /*
  * Adapted from https://www.mail-archive.com/harfbuzz@lists.freedesktop.org/msg06538.html
  * by Konstantin Ritt.
+ *
+ * HarfBuzz added GDI support after this code was done, and may not have been enabled
+ * in the build, so this now becomes the fallback method used to create a hb_face_t if
+ * HarfBuzz was neither built with DirectWrite nor GDI support.
  */
+#if !defined (USE_HB_DWRITE) && !defined (USE_HB_GDI)
 static hb_blob_t *
 hfont_reference_table (hb_face_t *face, hb_tag_t tag, void *user_data)
 {
@@ -1261,25 +1310,117 @@ hfont_reference_table (hb_face_t *face, hb_tag_t tag, void *user_data)
   SelectObject (hdc, old_hfont);
   return hb_blob_create (buf, size, HB_MEMORY_MODE_READONLY, buf, g_free);
 }
+#endif
 
 static hb_font_t *
 pango_win32_font_create_hb_font (PangoFont *font)
 {
   PangoWin32Font *win32font = (PangoWin32Font *)font;
-  HFONT hfont;
   hb_face_t *face = NULL;
   hb_font_t *hb_font = NULL;
 
   g_return_val_if_fail (font != NULL, NULL);
 
+#ifdef USE_HB_DWRITE
+  face = pango_win32_font_create_hb_face_dwrite (win32font);
+#else
+  HFONT hfont;
+
   hfont = _pango_win32_font_get_hfont (font);
 
-  /* We are *not* allowed to destroy the HFONT here ! */
+#ifdef USE_HB_GDI
+  face = hb_gdi_face_create (hfont);
+#else
   face = hb_face_create_for_tables (hfont_reference_table, (void *)hfont, NULL);
+#endif
+
+#endif
 
   hb_font = hb_font_create (face);
   hb_font_set_scale (hb_font, win32font->size, win32font->size);
+
+  if (win32font->variations)
+    {
+      unsigned int n_axes;
+
+      n_axes = hb_ot_var_get_axis_infos (face, 0, NULL, NULL);
+      if (n_axes > 0)
+        {
+          hb_ot_var_axis_info_t *axes;
+          float *coords;
+
+          axes = g_newa (hb_ot_var_axis_info_t, n_axes);
+          coords = g_newa (float, n_axes);
+
+          hb_ot_var_get_axis_infos (face, 0, &n_axes, axes);
+          for (unsigned int i = 0; i < n_axes; i++)
+            coords[axes[i].axis_index] = axes[i].default_value;
+
+          pango_parse_variations (win32font->variations, axes, n_axes, coords);
+
+          hb_font_set_var_coords_design (hb_font, coords, n_axes);
+        }
+    }
   hb_face_destroy (face);
 
   return hb_font;
 }
+
+gpointer
+_pango_win32_copy_cmap (gpointer cmap, guint16 cmap_format)
+{
+  if (!cmap)
+    return NULL;
+
+  if (cmap_format == 12)
+    {
+      struct format_12_cmap *new_table;
+      struct format_12_cmap *old_table;
+      guint32 *tbl, *tbl_end;
+
+      old_table = (struct format_12_cmap *) cmap;
+
+      new_table = g_malloc (old_table->length);
+      memcpy (old_table, new_table, sizeof (struct format_12_cmap));
+
+      tbl_end = (guint32 *) ((char *) new_table + new_table->length);
+      tbl = new_table->groups;
+
+      while (tbl < tbl_end)
+        {
+          *tbl = GUINT32_FROM_BE (*tbl);
+          tbl++;
+        }
+
+      return new_table;
+    }
+  else if (cmap_format == 4)
+    {
+      struct format_4_cmap *new_table;
+      struct format_4_cmap *old_table;
+      guint16 *tbl, *tbl_end;
+
+      old_table = (struct format_4_cmap *) cmap;
+
+      new_table = g_malloc (old_table->length);
+      memcpy (old_table, new_table, sizeof (struct format_4_cmap));
+
+      tbl_end = (guint16 *)((char *) new_table + new_table->length);
+      tbl = &new_table->reserved;
+
+      while (tbl < tbl_end)
+        {
+          *tbl = GUINT16_FROM_BE (*tbl);
+          tbl++;
+        }
+
+      return new_table;
+    }
+  else
+    {
+      /* got non-null cmap but unknown format, it shouldn't happen */
+      g_assert_not_reached ();
+    }
+
+  return NULL;
+}
